@@ -15,12 +15,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <KernelLog.h>
+#include <FreeNOS/User.h>
 #include "Init.h"
 
 int main(int argc, char **argv)
 {
+    SystemInformation info;
+
+    // Only run on core0
+    if (info.coreId != 0)
+        return 0;
+
     Init app(argc, argv);
-    KernelLog log;
     return app.run();
 }
