@@ -2,14 +2,14 @@
 # Use serial port as console.
 #
 /server/serial/server &
-write /sys/mountwait /dev/serial
+/bin/mount --wait=/dev/serial
 stdio /dev/serial/serial0/io /dev/serial/serial0/io
 
 #
 # System Servers and Drivers.
 #
-/server/usb/controller/server &
-/server/usb/hub/server &
+#/server/usb/controller/server &
+#/server/usb/hub/server &
 /server/filesystem/tmp/server /tmp &
 /server/network/loopback/server &
 
