@@ -5,7 +5,7 @@
 /server/ps2/server &
 /server/video/server &
 /server/terminal/server &
-write /sys/mountwait /console
+/bin/mount --wait=/console
 stdio /console/tty0 /console/tty0
 
 #
@@ -13,13 +13,12 @@ stdio /console/tty0 /console/tty0
 #
 /server/time/server &
 /server/filesystem/tmp/server /tmp &
-/server/network/loopback/server &
 
 #
 # Serial console
 #
 /server/serial/server &
-write /sys/mountwait /dev/serial
+/bin/mount --wait=/dev/serial
 
 # This ensures we wait until all cores
 # are booted by the CoreServer.
