@@ -15,8 +15,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <Log.h>
+#include <FreeNOS/System.h>
 #include <FreeNOS/ProcessManager.h>
+#include <Log.h>
 #include "PrivExec.h"
 
 API::Result PrivExecHandler(PrivOperation op, Address addr)
@@ -26,24 +27,26 @@ API::Result PrivExecHandler(PrivOperation op, Address addr)
     switch (op)
     {
     case Idle: {
-        ProcessManager *procs = Kernel::instance->getProcessManager();
+        ProcessManager *procs = Kernel::instance()->getProcessManager();
         procs->setIdle(procs->current());
         return API::Success;
     }
 
-    case Reboot:
+    case RebootSystem:
         cpu_reboot();
         while (true) ;
 
-    case Shutdown:
+    case ShutdownSystem:
         cpu_shutdown();
         return API::Success;
 
     case WriteConsole:
-        if (Log::instance)
-        {
-            (*Log::instance) << (char *)addr;
-        }
+        assert(Log::instance() != ZERO);
+        (*Log::instance()) << (char *)addr;
+        return API::Success;
+
+    case Panic:
+        FATAL("panic in PID " << Kernel::instance()->getProcessManager()->current()->getID());
         return API::Success;
 
     default:
