@@ -23,7 +23,6 @@
 #include <List.h>
 #include <MemoryMap.h>
 #include <Timer.h>
-#include <Index.h>
 #include "ProcessShares.h"
 
 /** @see IPCMessage.h. */
@@ -171,11 +170,9 @@ class Process
     /**
      * Prevent process from sleeping.
      *
-     * @param ignorePendingSleep True to ignore any (just) pending sleep of this Process
-     *
      * @return Result code
      */
-    Result wakeup(bool ignorePendingSleep = false);
+    Result wakeup();
 
     /**
      * Stops the process for executing until woken up
@@ -201,7 +198,7 @@ class Process
      *
      * @return Result code
      */
-    Result raiseEvent(struct ProcessEvent *event);
+    Result raiseEvent(const struct ProcessEvent *event);
 
     /**
      * Get sleep timer.
