@@ -22,6 +22,7 @@
 #include <MemoryMap.h>
 #include <Vector.h>
 #include <List.h>
+#include <Queue.h>
 #include "Process.h"
 
 /* Forward declarations */
@@ -80,10 +81,10 @@ class ProcessManager
      *
      * @return Process pointer on success or ZERO on failure
      */
-    Process * create(Address entry,
+    Process * create(const Address entry,
                      const MemoryMap &map,
-                     bool readyToRun = false,
-                     bool privileged = false);
+                     const bool readyToRun = false,
+                     const bool privileged = false);
 
     /**
      * Retrieve a Process by it's ID.
@@ -92,12 +93,12 @@ class ProcessManager
      *
      * @return Pointer to the appropriate process or ZERO if not found.
      */
-    Process * get(ProcessID id);
+    Process * get(const ProcessID id);
 
     /**
      * Remove a Process.
      */
-    void remove(Process *proc, uint exitStatus = 0);
+    void remove(Process *proc, const uint exitStatus = 0);
 
     /**
      * Schedule next process to run.
@@ -123,7 +124,7 @@ class ProcessManager
      *
      * @return Result code
      */
-    Result sleep(const Timer::Info *timer = 0, bool ignoreWakeups = false);
+    Result sleep(const Timer::Info *timer = 0, const bool ignoreWakeups = false);
 
     /**
      * Take Process out of Sleep state and mark ready for execution.
@@ -142,7 +143,7 @@ class ProcessManager
      *
      * @return Result code
      */
-    Result raiseEvent(Process *proc, struct ProcessEvent *event);
+    Result raiseEvent(Process *proc, const struct ProcessEvent *event);
 
     /**
      * Register an interrupt notification for a Process.
@@ -152,7 +153,7 @@ class ProcessManager
      *
      * @return Result code
      */
-    Result registerInterruptNotify(Process *proc, u32 vector);
+    Result registerInterruptNotify(Process *proc, const u32 vector);
 
     /**
      * Remove all interrupt notifications for a Process
@@ -170,7 +171,7 @@ class ProcessManager
      *
      * @return Result code
      */
-    Result interruptNotify(u32 vector);
+    Result interruptNotify(const u32 vector);
 
     /**
      * Set the idle process.
@@ -191,6 +192,28 @@ class ProcessManager
      */
     Vector<Process *> * getProcessTable();
 
+  private:
+
+    /**
+     * Place the given process on the Schedule queue
+     *
+     * @param proc Process pointer
+     * @param ignoreState True to not check for the Process state prior to dequeue.
+     *
+     * @return Result code
+     */
+    Result enqueueProcess(Process *proc, const bool ignoreState = false);
+
+    /**
+     * Remove the given process on the Schedule queue
+     *
+     * @param proc Process pointer
+     * @param ignoreState True to not check for the Process state prior to dequeue.
+     *
+     * @return Result code
+     */
+    Result dequeueProcess(Process *proc, const bool ignoreState = false) const;
+
   private:
 
     /** All known Processes. */
@@ -205,8 +228,8 @@ class ProcessManager
     /** Idle process */
     Process *m_idle;
 
-    /** Next timer */
-    Timer::Info m_nextSleepTimer;
+    /** Queue with sleeping processes waiting for a Timer to expire. */
+    Queue<Process *, MAX_PROCS> m_sleepTimerQueue;
 
     /** Interrupt notification list */
     Vector<List<Process *> *> m_interruptNotifyList;
