@@ -16,6 +16,7 @@
  */
 
 #include <FreeNOS/System.h>
+#include <MemoryContext.h>
 #include <MemoryChannel.h>
 #include <List.h>
 #include <ListIterator.h>
@@ -31,7 +32,7 @@ ProcessShares::ProcessShares(ProcessID pid)
 
 ProcessShares::~ProcessShares()
 {
-    ProcessManager *procs = Kernel::instance->getProcessManager();
+    ProcessManager *procs = Kernel::instance()->getProcessManager();
     List<ProcessID> pids;
 
     // Make a list of unique process IDs which
@@ -39,7 +40,7 @@ ProcessShares::~ProcessShares()
     Size size = m_shares.size();
     for (Size i = 0; i < size; i++)
     {
-        MemoryShare *sh = (MemoryShare *) m_shares.get(i);
+        MemoryShare *sh = m_shares.get(i);
         if (sh)
         {
             if (!pids.contains(sh->pid))
@@ -130,7 +131,8 @@ ProcessShares::Result ProcessShares::createShare(ProcessID pid,
     }
 
     // insert into shares list
-    m_shares.insert(*share);
+    Size idx = 0;
+    m_shares.insert(idx, share);
     return Success;
 }
 
@@ -143,6 +145,7 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
     MemoryContext *remoteMem = instance.getMemoryContext();
     Arch::Cache cache;
     Allocator::Range allocPhys, allocVirt;
+    Size idx = 0;
 
     if (share->range.size == 0)
         return InvalidArgument;
@@ -173,7 +176,7 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
     allocPhys.size = share->range.size;
     allocPhys.alignment = PAGESIZE;
 
-    if (Kernel::instance->getAllocator()->allocate(allocPhys, allocVirt) != Allocator::Success)
+    if (Kernel::instance()->getAllocator()->allocate(allocPhys, allocVirt) != Allocator::Success)
     {
         ERROR("failed to allocate pages for MemoryShare");
         return OutOfMemory;
@@ -186,7 +189,7 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
 
     // Fill the local share object
     localShare->pid        = instance.getProcessID();
-    localShare->coreId     = Kernel::instance->getCoreInfo()->coreId;
+    localShare->coreId     = Kernel::instance()->getCoreInfo()->coreId;
     localShare->tagId      = share->tagId;
     localShare->range.phys = allocPhys.address;
     localShare->range.size = share->range.size;
@@ -195,7 +198,7 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
 
     // Map in the local process
     if (localMem->findFree(localShare->range.size, MemoryMap::UserShare, &localShare->range.virt) != MemoryContext::Success ||
-        localMem->mapRange(&localShare->range) != MemoryContext::Success)
+        localMem->mapRangeContiguous(&localShare->range) != MemoryContext::Success)
     {
         ERROR("failed to map MemoryShare in local process");
         delete localShare;
@@ -213,7 +216,7 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
 
     // Map in the remote process
     if (remoteMem->findFree(remoteShare->range.size, MemoryMap::UserShare, &remoteShare->range.virt) != MemoryContext::Success ||
-        remoteMem->mapRange(&remoteShare->range) != MemoryContext::Success)
+        remoteMem->mapRangeContiguous(&remoteShare->range) != MemoryContext::Success)
     {
         ERROR("failed to map MemoryShare in remote process");
         delete localShare;
@@ -221,11 +224,11 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
         return OutOfMemory;
     }
     // insert into shares list
-    m_shares.insert(*localShare);
-    instance.m_shares.insert(*remoteShare);
+    m_shares.insert(idx, localShare);
+    instance.m_shares.insert(idx, remoteShare);
 
     // raise event on the remote process
-    ProcessManager *procs = Kernel::instance->getProcessManager();
+    ProcessManager *procs = Kernel::instance()->getProcessManager();
     Process *proc = procs->get(instance.getProcessID());
     ProcessEvent event;
     event.type   = ShareCreated;
@@ -240,12 +243,12 @@ ProcessShares::Result ProcessShares::createShare(ProcessShares & instance,
 
 ProcessShares::Result ProcessShares::removeShares(ProcessID pid)
 {
-    Size size = m_shares.size();
+    const Size size = m_shares.size();
     MemoryShare *s = 0;
 
     for (Size i = 0; i < size; i++)
     {
-        if ((s = (MemoryShare *) m_shares.get(i)) != ZERO)
+        if ((s = m_shares.get(i)) != ZERO)
         {
             if (s->pid != pid)
                 continue;
@@ -269,16 +272,16 @@ ProcessShares::Result ProcessShares::releaseShare(MemoryShare *s, Size idx)
     // other process.
     if (s->attached)
     {
-        Process *proc = Kernel::instance->getProcessManager()->get(s->pid);
+        Process *proc = Kernel::instance()->getProcessManager()->get(s->pid);
         if (proc)
         {
             ProcessShares & shares = proc->getShares();
-            Size size = shares.m_shares.size();
+            const Size size = shares.m_shares.size();
 
             // Mark all process shares detached in the other process
             for (Size i = 0; i < size; i++)
             {
-                MemoryShare *otherShare = (MemoryShare *) shares.m_shares.get(i);
+                MemoryShare *otherShare = shares.m_shares.get(i);
                 if (otherShare)
                 {
                     assert(otherShare->coreId == coreInfo.coreId);
@@ -295,23 +298,32 @@ ProcessShares::Result ProcessShares::releaseShare(MemoryShare *s, Size idx)
     {
         // Only release physical memory pages if the other
         // process already detached earlier
-        Allocator *alloc = Kernel::instance->getAllocator();
+        Allocator *alloc = Kernel::instance()->getAllocator();
 
         for (Size i = 0; i < s->range.size; i += PAGESIZE)
             alloc->release(s->range.phys + i);
     }
+
     // Unmap the share
     m_memory->unmapRange(&s->range);
 
-    // Remove share from our administration
-    m_shares.remove(idx);
+    // Release the share object
     delete s;
-    return Success;
+
+    // Remove share from our administration
+    if (!m_shares.remove(idx))
+    {
+        return NotFound;
+    }
+    else
+    {
+        return Success;
+    }
 }
 
 ProcessShares::Result ProcessShares::readShare(MemoryShare *share)
 {
-    Size size = m_shares.size();
+    const Size size = m_shares.size();
     const MemoryShare *s = 0;
 
     for (Size i = 0; i < size; i++)
