@@ -51,11 +51,20 @@ ARMKernel::ARMKernel(CoreInfo *info)
 #endif
 
     // Allocate physical memory for the temporary stack.
-    // Temporary only allocate on boot core, since on secondary
-    // cores its not within the memory.phys area anyway.
+    //
+    // This is an area of 1MiB which must not be used. It is re-mapped on the
+    // secondary cores to an identity-mapped area on the boot core and is currently
+    // required in order to enable the early MMU and perform full kernel startup
+    // until the first program starts.
+    //
+    // If this area is re-used after the kernel started, the SplitAllocator::toVirtual()
+    // function will not translate properly, resulting in memory corruption.
     if (m_coreInfo->coreId == 0) {
         for (Size i = 0; i < (PAGESIZE*4); i += PAGESIZE)
             m_alloc->allocate(TMPSTACKADDR + i);
+    } else {
+        for (Size i = 0; i < MegaByte(1); i += PAGESIZE)
+            m_alloc->allocate(info->kernel.phys + TMPSTACKOFF + i);
     }
 }
 
@@ -65,7 +74,7 @@ void ARMKernel::interrupt(CPUState state)
     core.logException(&state);
 
     FATAL("core" << coreInfo.coreId << ": unhandled IRQ in procId = " <<
-           Kernel::instance->getProcessManager()->current()->getID());
+           Kernel::instance()->getProcessManager()->current()->getID());
 }
 
 void ARMKernel::undefinedInstruction(CPUState state)
@@ -74,7 +83,7 @@ void ARMKernel::undefinedInstruction(CPUState state)
     core.logException(&state);
 
     FATAL("core" << coreInfo.coreId << ": procId = " <<
-           Kernel::instance->getProcessManager()->current()->getID());
+           Kernel::instance()->getProcessManager()->current()->getID());
 }
 
 void ARMKernel::prefetchAbort(CPUState state)
@@ -83,7 +92,7 @@ void ARMKernel::prefetchAbort(CPUState state)
     core.logException(&state);
 
     FATAL("core" << coreInfo.coreId << ": procId = " <<
-           Kernel::instance->getProcessManager()->current()->getID());
+           Kernel::instance()->getProcessManager()->current()->getID());
 }
 
 void ARMKernel::dataAbort(CPUState state)
@@ -92,7 +101,7 @@ void ARMKernel::dataAbort(CPUState state)
     core.logException(&state);
 
     FATAL("core" << coreInfo.coreId << ": procId = " <<
-           Kernel::instance->getProcessManager()->current()->getID());
+           Kernel::instance()->getProcessManager()->current()->getID());
 }
 
 
@@ -102,19 +111,19 @@ void ARMKernel::reserved(CPUState state)
     core.logException(&state);
 
     FATAL("core" << coreInfo.coreId << ": procId = " <<
-           Kernel::instance->getProcessManager()->current()->getID());
+           Kernel::instance()->getProcessManager()->current()->getID());
 }
 
 void ARMKernel::trap(volatile CPUState state)
 {
-    ProcessManager *mgr = Kernel::instance->getProcessManager();
+    ProcessManager *mgr = Kernel::instance()->getProcessManager();
     ARMProcess *proc = (ARMProcess *) mgr->current(), *proc2;
     ProcessID procId = proc->getID();
 
     DEBUG("coreId = " << coreInfo.coreId << " procId = " << procId << " api = " << state.r0);
 
     // Execute the kernel call
-    u32 r = Kernel::instance->getAPI()->invoke(
+    u32 r = Kernel::instance()->getAPI()->invoke(
         (API::Number) state.r0,
                       state.r1,
                       state.r2,
