@@ -23,7 +23,7 @@
 #define MEMALIGN8 8
 
 static bool firstProcess = true;
-extern u8 svcStack[PAGESIZE];
+extern u8 svcStack[PAGESIZE * 4];
 
 ARMProcess::ARMProcess(ProcessID id, Address entry, bool privileged, const MemoryMap &map)
     : Process(id, entry, privileged, map)
@@ -36,7 +36,7 @@ Process::Result ARMProcess::initialize()
     Allocator::Range alloc_args;
 
     // Create MMU context
-    m_memoryContext = new ARMPaging(&m_map, Kernel::instance->getAllocator());
+    m_memoryContext = new ARMPaging(&m_map, Kernel::instance()->getAllocator());
     if (!m_memoryContext)
     {
         ERROR("failed to create memory context");
@@ -50,7 +50,7 @@ Process::Result ARMProcess::initialize()
     alloc_args.size = range.size;
     alloc_args.alignment = PAGESIZE;
 
-    if (Kernel::instance->getAllocator()->allocate(alloc_args) != Allocator::Success)
+    if (Kernel::instance()->getAllocator()->allocate(alloc_args) != Allocator::Success)
     {
         ERROR("failed to allocate user stack");
         return OutOfMemory;
@@ -58,7 +58,7 @@ Process::Result ARMProcess::initialize()
     range.phys = alloc_args.address;
 
     // Map User stack
-    if (m_memoryContext->mapRange(&range) != MemoryContext::Success)
+    if (m_memoryContext->mapRangeContiguous(&range) != MemoryContext::Success)
     {
         ERROR("failed to map user stack");
         return MemoryMapError;
@@ -107,11 +107,12 @@ void ARMProcess::execute(Process *previous)
     {
         firstProcess = false;
 
-        CPUState *ptr = ((CPUState *) (svcStack)) - 1;
+        // Kernel stacks are currently 16KiB (see ARMBoot.S)
+        CPUState *ptr = ((CPUState *) (svcStack + sizeof(svcStack))) - 1;
         MemoryBlock::copy(ptr, &m_cpuState, sizeof(*ptr));
 
         // Switch to the actual SVC stack and switch to usermode
-        asm volatile ("ldr sp, =svcStack\n"
+        asm volatile ("ldr sp, =(svcStack + (4096*4))\n"
                       "sub sp, sp, %0\n"
                       "ldr r0, =loadCoreState0\n"
                       "bx r0\n" : : "i" (sizeof(m_cpuState) - sizeof(m_cpuState.padding)) );
