@@ -37,7 +37,7 @@ Process::Result IntelProcess::initialize()
     u16 codeSel = m_privileged ? KERNEL_CS_SEL : USER_CS_SEL;
 
     // Create MMU context
-    m_memoryContext = new IntelPaging(&m_map, Kernel::instance->getAllocator());
+    m_memoryContext = new IntelPaging(&m_map, Kernel::instance()->getAllocator());
     if (!m_memoryContext)
     {
         ERROR("failed to create memory context");
@@ -51,7 +51,7 @@ Process::Result IntelProcess::initialize()
     allocPhys.size = range.size;
     allocPhys.alignment = PAGESIZE;
 
-    if (Kernel::instance->getAllocator()->allocate(allocPhys) != Allocator::Success)
+    if (Kernel::instance()->getAllocator()->allocate(allocPhys) != Allocator::Success)
     {
         ERROR("failed to allocate user stack");
         return OutOfMemory;
@@ -59,7 +59,7 @@ Process::Result IntelProcess::initialize()
     range.phys = allocPhys.address;
 
     // Map User stack
-    if (m_memoryContext->mapRange(&range) != MemoryContext::Success)
+    if (m_memoryContext->mapRangeContiguous(&range) != MemoryContext::Success) 
     {
         ERROR("failed to map user stack");
         return MemoryMapError;
@@ -71,7 +71,7 @@ Process::Result IntelProcess::initialize()
     allocPhys.size = KernelStackSize;
     allocPhys.alignment = PAGESIZE;
 
-    if (Kernel::instance->getAllocator()->allocate(allocPhys, allocVirt) != Allocator::Success)
+    if (Kernel::instance()->getAllocator()->allocate(allocPhys, allocVirt) != Allocator::Success)
     {
         ERROR("failed to allocate kernel stack");
         return OutOfMemory;
@@ -96,7 +96,7 @@ Process::Result IntelProcess::initialize()
     regs->irq.eip    = m_entry;
     regs->irq.cs     = codeSel;
     regs->irq.eflags = INTEL_EFLAGS_DEFAULT |
-                       INTEL_EFLAGS_IRQ;
+                       INTEL_EFLAGS_IRQ | (0x3 << 12);
     regs->irq.esp3   = userStack;
     regs->irq.ss3    = dataSel;
 
@@ -119,7 +119,7 @@ Process::Result IntelProcess::initialize()
 IntelProcess::~IntelProcess()
 {
     // Release the kernel stack memory page
-    SplitAllocator *alloc = Kernel::instance->getAllocator();
+    SplitAllocator *alloc = Kernel::instance()->getAllocator();
     alloc->release((Address)alloc->toPhysical(m_kernelStackBase) - KernelStackSize);
 }
 
