@@ -15,61 +15,49 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <FreeNOS/System.h>
+#include <FreeNOS/User.h>
 #include <HashIterator.h>
-#include <FileSystemMessage.h>
 #include "ChannelClient.h"
 #include "MemoryChannel.h"
 
 ChannelClient::ChannelClient()
-    : Singleton<ChannelClient>(this)
+    : StrictSingleton<ChannelClient>()
 {
-    m_registry = 0;
 }
 
 ChannelClient::~ChannelClient()
 {
 }
 
-ChannelRegistry * ChannelClient::getRegistry()
+ChannelRegistry & ChannelClient::getRegistry()
 {
     return m_registry;
 }
 
-ChannelClient::Result ChannelClient::setRegistry(ChannelRegistry *registry)
-{
-    m_registry = registry;
-    return Success;
-}
-
 ChannelClient::Result ChannelClient::initialize()
 {
     return Success;
 }
 
-ChannelClient::Result ChannelClient::connect(ProcessID pid, Size messageSize)
+ChannelClient::Result ChannelClient::connect(const ProcessID pid, const Size messageSize)
 {
     Address prodAddr, consAddr;
-    SystemInformation info;
+    const SystemInformation info;
 
     // Allocate consumer
-    MemoryChannel *cons = new MemoryChannel;
+    MemoryChannel *cons = new MemoryChannel(Channel::Consumer, messageSize);
     if (!cons)
     {
         return OutOfMemory;
     }
-    cons->setMessageSize(messageSize);
-    cons->setMode(Channel::Consumer);
 
     // Allocate producer
-    MemoryChannel *prod = new MemoryChannel;
+    MemoryChannel *prod = new MemoryChannel(Channel::Producer, messageSize);
     if (!prod)
     {
         delete cons;
         return OutOfMemory;
     }
-    prod->setMessageSize(messageSize);
-    prod->setMode(Channel::Producer);
 
     // Call VMShare to create shared memory mapping for MemoryChannel.
     ProcessShares::MemoryShare share;
@@ -82,7 +70,7 @@ ChannelClient::Result ChannelClient::connect(ProcessID pid, Size messageSize)
     share.range.access = Memory::User | Memory::Readable | Memory::Writable;
 
     // Create shared memory mapping
-    Error r = VMShare(pid, API::Create, &share);
+    const Error r = VMShare(pid, API::Create, &share);
     switch (r)
     {
         case API::Success:
@@ -120,14 +108,16 @@ ChannelClient::Result ChannelClient::connect(ProcessID pid, Size messageSize)
     }
 
     // Register channels
-    m_registry->registerConsumer(pid, cons);
-    m_registry->registerProducer(pid, prod);
+    m_registry.registerConsumer(pid, cons);
+    m_registry.registerProducer(pid, prod);
     return Success;
 }
 
-ChannelClient::Result ChannelClient::receiveAny(void *buffer, ProcessID *pid)
+ChannelClient::Result ChannelClient::receiveAny(void *buffer, const Size msgSize, ProcessID *pid)
 {
-    for (HashIterator<ProcessID, Channel *> i(m_registry->getConsumers()); i.hasCurrent(); i++)
+    assert(msgSize > 0);
+
+    for (HashIterator<ProcessID, Channel *> i(m_registry.getConsumers()); i.hasCurrent(); i++)
     {
         if (i.current()->read(buffer) == Channel::Success)
         {
@@ -138,26 +128,28 @@ ChannelClient::Result ChannelClient::receiveAny(void *buffer, ProcessID *pid)
     return NotFound;
 }
 
-ChannelClient::Result ChannelClient::sendRequest(ProcessID pid,
+ChannelClient::Result ChannelClient::sendRequest(const ProcessID pid,
                                                  void *buffer,
+                                                 const Size msgSize,
                                                  CallbackFunction *callback)
 {
     Request *req = 0;
     Size identifier = 0;
-    Channel *ch  = findProducer(pid);
+    Channel *ch = findProducer(pid, msgSize);
     if (!ch)
         return NotFound;
 
     // Find request object
     for (Size i = 0; i < m_requests.count(); i++)
     {
-        req = (Request *) m_requests.get(i);
+        req = m_requests.get(i);
         if (!req->active)
         {
             identifier = i;
             break;
         }
     }
+
     // Allocate new request object if none available
     if (!req || req->active)
     {
@@ -165,8 +157,13 @@ ChannelClient::Result ChannelClient::sendRequest(ProcessID pid,
         assert(req != NULL);
         req->message = (ChannelMessage *) new u8[ch->getMessageSize()];
         assert(req->message != NULL);
-        identifier   = m_requests.insert(*req);
+
+        if (!m_requests.insert(identifier, req))
+        {
+            return OutOfMemory;
+        }
     }
+
     // Fill request object
     MemoryBlock::copy(req->message, buffer, ch->getMessageSize());
     req->pid = pid;
@@ -183,19 +180,20 @@ ChannelClient::Result ChannelClient::sendRequest(ProcessID pid,
         req->active = false;
         return IOError;
     }
+
     // Wakeup the receiver
     ProcessCtl(pid, Resume, 0);
     return Success;
 }
 
-ChannelClient::Result ChannelClient::processResponse(ProcessID pid,
+ChannelClient::Result ChannelClient::processResponse(const ProcessID pid,
                                                      ChannelMessage *msg)
 {
-    Size count = m_requests.count();
+    const Size count = m_requests.count();
 
     for (Size i = 0; i < count; i++)
     {
-        Request *req = (Request *) m_requests.get(i);
+        Request *req = m_requests.get(i);
 
         if (req->active &&
             req->pid == pid &&
@@ -210,37 +208,35 @@ ChannelClient::Result ChannelClient::processResponse(ProcessID pid,
 }
 
 
-Channel * ChannelClient::findConsumer(ProcessID pid)
+Channel * ChannelClient::findConsumer(const ProcessID pid, const Size msgSize)
 {
-    Result r;
-    Channel *ch = m_registry->getConsumer(pid);
+    Channel *ch = m_registry.getConsumer(pid);
     if (ch)
         return ch;
 
     // Try to connect
-    if ((r = connect(pid)) != Success)
+    if (connect(pid, msgSize) != Success)
         return ZERO;
 
-    return m_registry->getConsumer(pid);
+    return m_registry.getConsumer(pid);
 }
 
-Channel * ChannelClient::findProducer(ProcessID pid)
+Channel * ChannelClient::findProducer(const ProcessID pid, const Size msgSize)
 {
-    Result r;
-    Channel *ch = m_registry->getProducer(pid);
+    Channel *ch = m_registry.getProducer(pid);
     if (ch)
         return ch;
 
     // Try to connect
-    if ((r = connect(pid)) != Success)
+    if (connect(pid, msgSize) != Success)
         return ZERO;
 
-    return m_registry->getProducer(pid);
+    return m_registry.getProducer(pid);
 }
 
-ChannelClient::Result ChannelClient::syncReceiveFrom(void *buffer, ProcessID pid)
+ChannelClient::Result ChannelClient::syncReceiveFrom(void *buffer, const Size msgSize, const ProcessID pid)
 {
-    Channel *ch = findConsumer(pid);
+    Channel *ch = findConsumer(pid, msgSize);
     if (!ch)
         return NotFound;
 
@@ -250,9 +246,9 @@ ChannelClient::Result ChannelClient::syncReceiveFrom(void *buffer, ProcessID pid
     return Success;
 }
 
-ChannelClient::Result ChannelClient::syncSendTo(void *buffer, ProcessID pid)
+ChannelClient::Result ChannelClient::syncSendTo(const void *buffer, const Size msgSize, const ProcessID pid)
 {
-    Channel *ch = findProducer(pid);
+    Channel *ch = findProducer(pid, msgSize);
     if (!ch)
         return NotFound;
 
@@ -276,11 +272,11 @@ ChannelClient::Result ChannelClient::syncSendTo(void *buffer, ProcessID pid)
     return IOError;
 }
 
-ChannelClient::Result ChannelClient::syncSendReceive(void *buffer, ProcessID pid)
+ChannelClient::Result ChannelClient::syncSendReceive(void *buffer, const Size msgSize, const ProcessID pid)
 {
-    Result r = syncSendTo(buffer, pid);
-    if (r != Success)
-        return r;
+    const Result result = syncSendTo(buffer, msgSize, pid);
+    if (result != Success)
+        return result;
 
-    return syncReceiveFrom(buffer, pid);
+    return syncReceiveFrom(buffer, msgSize, pid);
 }
