@@ -18,11 +18,12 @@
 #ifndef __LIBIPC_CHANNELSERVER_H
 #define __LIBIPC_CHANNELSERVER_H
 
-#include <FreeNOS/System.h>
+#include <FreeNOS/User.h>
 #include <FreeNOS/ProcessEvent.h>
 #include <FreeNOS/ProcessShares.h>
 #include <HashIterator.h>
 #include <Timer.h>
+#include <Vector.h>
 #include "MemoryChannel.h"
 #include "ChannelClient.h"
 #include "ChannelRegistry.h"
@@ -43,18 +44,30 @@ template <class Func> struct MessageHandler
     /**
      * Constructor function.
      *
-     * @param f Function to execute.
-     * @param r Send a reply?
+     * @param func Function to execute.
+     * @param reply True to send a reply for this message
      */
-    MessageHandler(Func f, bool r) : exec(f), sendReply(r)
+    MessageHandler(const Func func, const bool reply)
+        : exec(func)
+        , sendReply(reply)
     {
     }
 
+    const bool operator == (const struct MessageHandler<Func> & h) const
+    {
+        return false;
+    }
+
+    const bool operator != (const struct MessageHandler<Func> & h) const
+    {
+        return false;
+    }
+
     /** Handler function. */
-    Func exec;
+    const Func exec;
 
     /** Whether to send a reply or not. */
-    bool sendReply;
+    const bool sendReply;
 };
 
 /**
@@ -64,6 +77,11 @@ template <class Func> struct MessageHandler
  */
 template <class Base, class MsgType> class ChannelServer
 {
+  private:
+
+    /** Maximum number of IPC/IRQ handlers. */
+    static const Size MaximumHandlerCount = 255u;
+
   protected:
 
     /** Member function pointer inside Base, to handle IPC messages. */
@@ -85,26 +103,27 @@ template <class Base, class MsgType> class ChannelServer
         IOError,
     };
 
+  public:
+
     /**
      * Constructor function.
-     *
-     * @param num Number of message handlers to support.
      */
-    ChannelServer(Base *inst, Size num = 32)
-        : m_sendReply(true), m_instance(inst)
+    ChannelServer(Base *inst)
+        : m_instance(inst)
+        , m_client(ChannelClient::instance())
+        , m_registry(m_client->getRegistry())
+        , m_kernelEvent(Channel::Consumer, sizeof(ProcessEvent))
+        , m_ipcHandlers()
+        , m_irqHandlers()
     {
         m_self = ProcessCtl(SELF, GetPID, 0);
 
-        m_client   = ChannelClient::instance;
-        m_registry = m_client->getRegistry();
-
-        m_ipcHandlers = new Vector<MessageHandler<IPCHandlerFunction> *>(num);
-        m_ipcHandlers->fill(ZERO);
-        m_irqHandlers = new Vector<MessageHandler<IRQHandlerFunction> *>(num);
-        m_irqHandlers->fill(ZERO);
+        // Reset timeout values
+        m_expiry.frequency = 0;
+        m_expiry.ticks = 0;
 
         // Setup kernel event channel
-        SystemInformation info;
+        const SystemInformation info;
         ProcessShares::MemoryShare share;
         share.pid    = KERNEL_PID;
         share.coreId = info.coreId;
@@ -112,12 +131,10 @@ template <class Base, class MsgType> class ChannelServer
 
         if (VMShare(SELF, API::Read, &share) != API::Success)
         {
-            ERROR("failed to get kernel event channel");
+            FATAL("failed to get kernel event channel");
         }
         else
         {
-            m_kernelEvent.setMode(Channel::Consumer);
-            m_kernelEvent.setMessageSize(sizeof(ProcessEvent));
             m_kernelEvent.setVirtual(share.range.virt,
                                      share.range.virt + PAGESIZE);
         }
@@ -128,10 +145,8 @@ template <class Base, class MsgType> class ChannelServer
      */
     virtual ~ChannelServer()
     {
-        delete m_client;
-        delete m_registry;
-        delete m_ipcHandlers;
-        delete m_irqHandlers;
+        m_ipcHandlers.deleteAll();
+        m_irqHandlers.deleteAll();
     }
 
     /**
@@ -144,48 +159,36 @@ template <class Base, class MsgType> class ChannelServer
         // Enter loop
         while (true)
         {
-            // Reset
-            m_sendReply = true;
-
-            // Process kernel events
-            readKernelEvents();
-
-            // Process user messages
-            readChannels();
-
-            // Retry requests until all served (EAGAIN or return value)
-            retryAllRequests();
-
-            // Sleep with timeout or return in case the process is
-            // woken up by an external (wakeup) interrupt.
-            DEBUG("EnterSleep");
-            Address expiry = 0;
-
-            if (m_expiry.frequency)
-                expiry = (Address) &m_expiry;
-
-            Error r = ProcessCtl(SELF, EnterSleep, expiry, (Address) &m_time);
-            DEBUG("EnterSleep returned: " << (int)r);
-
-            // Check for sleep timeout
-            if (m_expiry.frequency)
-            {
-                if (ProcessCtl(SELF, InfoTimer, (Address) &m_time) != API::Success)
-                {
-                    ERROR("failed to retrieve system timer");
-                }
-                else if (m_expiry.ticks < m_time.ticks)
-                {
-                    m_expiry.frequency = 0;
-                    timeout();
-                }
-            }
+            processAll();
+            sleepUntilWakeup();
         }
 
         // Satify compiler
         return 0;
     }
 
+    /**
+     * Set a sleep timeout
+     *
+     * @param msec Milliseconds to sleep (approximately)
+     */
+    void setTimeout(const uint msec)
+    {
+        DEBUG("msec = " << msec);
+
+        if (ProcessCtl(SELF, InfoTimer, (Address) &m_time) != API::Success)
+        {
+            ERROR("failed to retrieve system timer info");
+            return;
+        }
+
+        const Size msecPerTick = 1000 / m_time.frequency;
+        m_expiry.frequency = m_time.frequency;
+        m_expiry.ticks     = m_time.ticks + ((msec / msecPerTick) + 1);
+    }
+
+  protected:
+
     /**
      * Register a new IPC message action handler.
      *
@@ -193,9 +196,9 @@ template <class Base, class MsgType> class ChannelServer
      * @param h Handler to execute.
      * @param r Does the handler need to send a reply (per default) ?
      */
-    void addIPCHandler(Size slot, IPCHandlerFunction h, bool sendReply = true)
+    void addIPCHandler(const Size slot, IPCHandlerFunction h, const bool sendReply = true)
     {
-        m_ipcHandlers->insert(slot, new MessageHandler<IPCHandlerFunction>(h, sendReply));
+        m_ipcHandlers.insertAt(slot, new MessageHandler<IPCHandlerFunction>(h, sendReply));
     }
 
     /**
@@ -204,9 +207,9 @@ template <class Base, class MsgType> class ChannelServer
      * @param slot Vector value to trigger h.
      * @param h Handler to execute.
      */
-    void addIRQHandler(Size slot, IRQHandlerFunction h)
+    void addIRQHandler(const Size slot, IRQHandlerFunction h)
     {
-        m_irqHandlers->insert(slot, new MessageHandler<IRQHandlerFunction>(h, false));
+        m_irqHandlers.insertAt(slot, new MessageHandler<IRQHandlerFunction>(h, false));
     }
 
     /**
@@ -215,6 +218,8 @@ template <class Base, class MsgType> class ChannelServer
     virtual void timeout()
     {
         DEBUG("");
+
+        retryAllRequests();
     }
 
     /**
@@ -228,27 +233,62 @@ template <class Base, class MsgType> class ChannelServer
     }
 
     /**
-     * Set a sleep timeout
-     *
-     * @param msec Milliseconds to sleep (approximately)
+     * Keep retrying requests until all served
      */
-    void setTimeout(uint msec)
+    void retryAllRequests()
     {
-        DEBUG("msec = " << msec);
-
-        if (ProcessCtl(SELF, InfoTimer, (Address) &m_time) != API::Success)
-        {
-            ERROR("failed to retrieve system timer info");
-            return;
-        }
-
-        Size msecPerTick = 1000 / m_time.frequency;
-        m_expiry.frequency = m_time.frequency;
-        m_expiry.ticks     = m_time.ticks + ((msec / msecPerTick) + 1);
+        while (m_instance->retryRequests())
+            ;
     }
 
   private:
 
+    /**
+     * Process all current events and channels.
+     */
+    inline void processAll()
+    {
+        // Process kernel events
+        readKernelEvents();
+
+        // Process user messages
+        readChannels();
+
+        // Retry requests until all served (EAGAIN or return value)
+        retryAllRequests();
+    }
+
+    /**
+     * Let this process sleep until more events are raised.
+     */
+    inline void sleepUntilWakeup()
+    {
+        // Sleep with timeout or return in case the process is
+        // woken up by an external (wakeup) interrupt.
+        DEBUG("EnterSleep");
+        Address expiry = 0;
+
+        if (m_expiry.frequency)
+            expiry = (Address) &m_expiry;
+
+        const Error r = ProcessCtl(SELF, EnterSleep, expiry, (Address) (m_expiry.frequency ? &m_time : 0));
+        DEBUG("EnterSleep returned: " << (int)r);
+
+        // Check for sleep timeout
+        if (m_expiry.frequency)
+        {
+            if (ProcessCtl(SELF, InfoTimer, (Address) &m_time) != API::Success)
+            {
+                ERROR("failed to retrieve system timer");
+            }
+            else if (m_expiry.ticks < m_time.ticks)
+            {
+                m_expiry.frequency = 0;
+                timeout();
+            }
+        }
+    }
+
     /**
      * Accept new channel connection.
      *
@@ -257,28 +297,24 @@ template <class Base, class MsgType> class ChannelServer
      *
      * @return Result code
      */
-    Result accept(ProcessID pid, Memory::Range range)
+    Result accept(const ProcessID pid, const Memory::Range range)
     {
         // Create consumer
-        if (!m_registry->getConsumer(pid))
+        if (!m_registry.getConsumer(pid))
         {
-            MemoryChannel *consumer = new MemoryChannel;
+            MemoryChannel *consumer = new MemoryChannel(Channel::Consumer, sizeof(MsgType));
             assert(consumer != NULL);
-            consumer->setMode(Channel::Consumer);
-            consumer->setMessageSize(sizeof(MsgType));
             consumer->setVirtual(range.virt, range.virt + PAGESIZE);
-            m_registry->registerConsumer(pid, consumer);
+            m_registry.registerConsumer(pid, consumer);
         }
         // Create producer
-        if (!m_registry->getProducer(pid))
+        if (!m_registry.getProducer(pid))
         {
-            MemoryChannel *producer = new MemoryChannel;
+            MemoryChannel *producer = new MemoryChannel(Channel::Producer, sizeof(MsgType));
             assert(producer != NULL);
-            producer->setMode(Channel::Producer);
-            producer->setMessageSize(sizeof(MsgType));
             producer->setVirtual(range.virt + (PAGESIZE*2),
                                  range.virt + (PAGESIZE*3));
-            m_registry->registerProducer(pid, producer);
+            m_registry.registerProducer(pid, producer);
         }
         // Done
         return Success;
@@ -311,23 +347,28 @@ template <class Base, class MsgType> class ChannelServer
                 {
                     DEBUG(m_self << ": interrupt: " << event.number);
 
-                    if (m_irqHandlers->at(event.number))
+                    const MessageHandler<IRQHandlerFunction> *h = m_irqHandlers.get(event.number);
+                    if (h)
                     {
-                        (m_instance->*(m_irqHandlers->at(event.number))->exec) (event.number);
+                        (m_instance->*h->exec) (event.number);
+                    }
+                    else
+                    {
+                        ERROR(m_self << ": unhandled IRQ raised: " << event.number);
                     }
                     break;
                 }
                 case ProcessTerminated:
                 {
                     DEBUG(m_self << ": process terminated: PID " << event.number);
-                    result = m_registry->unregisterConsumer(event.number);
+                    result = m_registry.unregisterConsumer(event.number);
                     if (result != ChannelRegistry::Success)
                     {
                         ERROR("failed to unregister consumer for PID " <<
                                event.number << ": " << (int)result);
                     }
 
-                    result = m_registry->unregisterProducer(event.number);
+                    result = m_registry.unregisterProducer(event.number);
                     if (result != ChannelRegistry::Success)
                     {
                         ERROR("failed to unregister producer for PID " <<
@@ -335,7 +376,7 @@ template <class Base, class MsgType> class ChannelServer
                     }
 
                     // cleanup the VMShare area now for that process
-                    API::Result shareResult = VMShare(event.number, API::Delete, ZERO);
+                    const API::Result shareResult = VMShare(event.number, API::Delete, ZERO);
                     if (shareResult != API::Success)
                     {
                         ERROR("failed to remove shares with VMShare for PID " <<
@@ -361,7 +402,7 @@ template <class Base, class MsgType> class ChannelServer
         MsgType msg;
 
         // Try to receive message on each consumer channel
-        for (HashIterator<ProcessID, Channel *> i(m_registry->getConsumers()); i.hasCurrent(); i++)
+        for (HashIterator<ProcessID, Channel *> i(m_registry.getConsumers()); i.hasCurrent(); i++)
         {
             Channel *ch = i.current();
             DEBUG(m_self << ": trying to receive from PID " << i.key());
@@ -382,25 +423,32 @@ template <class Base, class MsgType> class ChannelServer
                     }
                 }
                 // Message is a request to us
-                else if (m_ipcHandlers->at(msg.action))
+                else
                 {
-                    m_sendReply = m_ipcHandlers->at(msg.action)->sendReply;
-                    (m_instance->*(m_ipcHandlers->at(msg.action))->exec) (&msg);
-
-                    // Send reply
-                    if (m_sendReply)
+                    const MessageHandler<IPCHandlerFunction> *h = m_ipcHandlers.get(msg.action);
+                    if (h)
                     {
-                        Channel *ch = m_registry->getProducer(i.key());
-                        if (!ch)
+                        (m_instance->*h->exec) (&msg);
+
+                        // Send reply
+                        if (h->sendReply)
                         {
-                            ERROR(m_self << ": no producer channel found for PID: " << i.key());
+                            Channel *ch = m_registry.getProducer(i.key());
+                            if (!ch)
+                            {
+                                ERROR(m_self << ": no producer channel found for PID: " << i.key());
+                            }
+                            else if (ch->write(&msg) != Channel::Success)
+                            {
+                                ERROR(m_self << ": failed to send reply message to PID: " << i.key());
+                            }
+                            else
+                                ProcessCtl(i.key(), Resume, 0);
                         }
-                        else if (ch->write(&msg) != Channel::Success)
-                        {
-                            ERROR(m_self << ": failed to send reply message to PID: " << i.key());
-                        }
-                        else
-                            ProcessCtl(i.key(), Resume, 0);
+                    }
+                    else
+                    {
+                        ERROR(m_self << ": invalid action " << (int)msg.action << " from PID " << i.key());
                     }
                 }
             }
@@ -408,46 +456,34 @@ template <class Base, class MsgType> class ChannelServer
         return Success;
     }
 
-    /**
-     * Keep retrying requests until all served
-     */
-    void retryAllRequests()
-    {
-        while (m_instance->retryRequests())
-            ;
-    }
-
   protected:
 
-    /** Contains registered channels */
-    ChannelRegistry *m_registry;
-
-    /** Client for sending replies */
-    ChannelClient *m_client;
-
-    /** Kernel event channel */
-    MemoryChannel m_kernelEvent;
-
-    /** Should we send a reply message? */
-    bool m_sendReply;
-
-    /** IPC handler functions. */
-    Vector<MessageHandler<IPCHandlerFunction> *> *m_ipcHandlers;
-
-    /** IRQ handler functions. */
-    Vector<MessageHandler<IRQHandlerFunction> *> *m_irqHandlers;
-
     /** Server object instance. */
     Base *m_instance;
 
+    /** Client for sending replies */
+    ChannelClient *m_client;
+
+    /** Contains registered channels */
+    ChannelRegistry &m_registry;
+
+    /** Kernel event channel */
+    MemoryChannel m_kernelEvent;
+
+    /** IPC handler functions. */
+    Index<MessageHandler<IPCHandlerFunction>, MaximumHandlerCount> m_ipcHandlers;
+
+    /** IRQ handler functions. */
+    Index<MessageHandler<IRQHandlerFunction>, MaximumHandlerCount> m_irqHandlers;
+
     /** ProcessID of ourselves */
     ProcessID m_self;
 
+  private:
+
     /** System timer value */
     Timer::Info m_time;
 
-  private:
-
     /** System timer expiration value */
     Timer::Info m_expiry;
 };
