@@ -15,13 +15,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <FreeNOS/System.h>
-#include <Index.h>
 #include <MemoryChannel.h>
+#include <Index.h>
 #include "mpi.h"
 #include "MPIMessage.h"
 
-extern Index<MemoryChannel> *writeChannel;
+extern Index<MemoryChannel, MPI_MAX_CHANNELS> *writeChannel;
 
 int MPI_Send(const void *buf,
              int count,
@@ -36,7 +35,7 @@ int MPI_Send(const void *buf,
     if (datatype != MPI_INT)
         return MPI_ERR_UNSUPPORTED_DATAREP;
 
-    if (!(ch = (MemoryChannel *) writeChannel->get(dest)))
+    if (!(ch = writeChannel->get(dest)))
         return MPI_ERR_RANK;
 
     for (int i = 0; i < count; i++)
