@@ -15,40 +15,45 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "Assert.h"
 #include "BitArray.h"
 #include "MemoryBlock.h"
 
-BitArray::BitArray(Size size, u8 *array)
+BitArray::BitArray(const Size bitCount, u8 *array)
 {
-    m_array = array ? array : new u8[BITS_TO_BYTES(size)];
-    m_allocated = array == ZERO;
-    m_size  = size;
-    m_set   = 0;
+    m_array = array ? array : new u8[calculateBitmapSize(bitCount)];
+    m_allocated = (array == ZERO);
+    m_bitCount  = bitCount;
+    m_set = 0;
+
     clear();
 }
 
 BitArray::~BitArray()
 {
     if (m_allocated)
+    {
         delete[] m_array;
+    }
 }
 
 Size BitArray::size() const
 {
-    return m_size;
+    return m_bitCount;
 }
 
-Size BitArray::count(bool on) const
+Size BitArray::count(const bool on) const
 {
-    return on ? m_set : m_size - m_set;
+    return on ? m_set : m_bitCount - m_set;
 }
 
-void BitArray::set(Size bit, bool value)
+void BitArray::set(const Size bit, const bool value)
 {
-
     // Check if the bit is inside the array
-    if (bit >= m_size)
+    if (bit >= m_bitCount)
+    {
         return;
+    }
 
     // Check current value
     bool current = m_array[bit / 8] & (1 << (bit % 8));
@@ -69,43 +74,73 @@ void BitArray::set(Size bit, bool value)
     }
 }
 
-void BitArray::unset(Size bit)
+void BitArray::unset(const Size bit)
 {
     set(bit, false);
 }
 
-bool BitArray::isSet(Size bit) const
+bool BitArray::isSet(const Size bit) const
 {
-    assert(bit < m_size);
+    assert(bit < m_bitCount);
 
     return m_array[bit / 8] & (1 << (bit % 8));
 }
 
-void BitArray::setRange(Size from, Size to)
+void BitArray::setRange(const Size from, const Size to)
 {
     for (Size i = from; i <= to; i++)
+    {
         set(i, true);
+    }
 }
 
-BitArray::Result BitArray::setNext(Size *bit, Size count, Size start, Size boundary)
+BitArray::Result BitArray::setNext(Size *bit,
+                                   const Size count,
+                                   const Size start,
+                                   const Size boundary)
 {
     Size from = 0, found = 0;
 
     // Loop BitArray for unset bits
-    for (Size i = start; i < m_size; i++)
+    for (Size i = start; i < m_bitCount;)
     {
-        if (!isSet(i))
+        // Try to fast-forward 32 bits to search
+        if (((u32 *)m_array)[i / 32] == 0xffffffff)
+        {
+            from = found = 0;
+
+            if (i & 31)
+                i += (32 - (i % 32));
+            else
+                i += 32;
+            continue;
+        }
+        // Try to fast-forward 8 bits to search
+        else if (m_array[i / 8] == 0xff)
+        {
+            from = found = 0;
+
+            if (i & 7)
+                i += (8 - (i % 8));
+            else
+                i += 8;
+            continue;
+        }
+        else if (!isSet(i))
         {
             // Remember this bit
             if (!found)
             {
-                if (i % boundary)
-                    continue;
-                from  = i;
-                found = 1;
+                if (!(i % boundary))
+                {
+                    from  = i;
+                    found = 1;
+                }
             }
             else
+            {
                 found++;
+            }
 
             // Are there enough contigious bits?
             if (found >= count)
@@ -119,6 +154,9 @@ BitArray::Result BitArray::setNext(Size *bit, Size count, Size start, Size bound
         {
             from = found = 0;
         }
+
+        // Move to the next bit
+        i++;
     }
     // No unset bits left!
     return OutOfMemory;
@@ -129,15 +167,19 @@ u8 * BitArray::array() const
     return m_array;
 }
 
-void BitArray::setArray(u8 *map, Size size)
+void BitArray::setArray(u8 *map, const Size bitCount)
 {
     // Set bits count
-    if (size)
-        m_size = size;
+    if (bitCount)
+    {
+        m_bitCount = bitCount;
+    }
 
     // Cleanup old array, if needed
     if (m_array && m_allocated)
+    {
         delete[] m_array;
+    }
 
     // Reassign to the new map
     m_array = map;
@@ -145,26 +187,40 @@ void BitArray::setArray(u8 *map, Size size)
     m_set   = 0;
 
     // Recalculate set bits
-    for (Size i = 0; i < m_size; i++)
+    for (Size i = 0; i < m_bitCount; i++)
+    {
         if (isSet(i))
+        {
             m_set++;
+        }
+    }
 }
 
 void BitArray::clear()
 {
     // Zero it
-    MemoryBlock::set(m_array, 0, BITS_TO_BYTES(m_size));
+    MemoryBlock::set(m_array, 0, calculateBitmapSize(m_bitCount));
 
     // Reset set count
     m_set = 0;
 }
 
-bool BitArray::operator[](Size bit) const
+bool BitArray::operator[](const Size bit) const
 {
     return isSet(bit);
 }
 
-bool BitArray::operator[](int bit) const
+bool BitArray::operator[](const int bit) const
 {
     return isSet(bit);
 }
+
+inline Size BitArray::calculateBitmapSize(const Size bitCount) const
+{
+    const Size bytes = bitCount / 8;
+
+    if (bitCount % 8)
+        return bytes + 1;
+    else
+        return bytes;
+}
