@@ -19,7 +19,6 @@
 #ifndef __SERVER_SERIAL_PL011_H
 #define __SERVER_SERIAL_PL011_H
 
-#include <FreeNOS/API.h>
 #include <FreeNOS/System.h>
 #include <Log.h>
 #include <Types.h>
@@ -50,18 +49,18 @@ class PL011 : public Device
     /**
      * @brief Initializes the UART.
      *
-     * @return Error status code.
+     * @return FileSystem::Error status code.
      */
-    virtual Error initialize();
+    virtual FileSystem::Error initialize();
 
     /**
      * Called when an interrupt has been triggered for this device.
      *
      * @param vector Vector number of the interrupt.
      *
-     * @return Error result code.
+     * @return FileSystem::Error result code.
      */
-    virtual Error interrupt(Size vector);
+    virtual FileSystem::Error interrupt(Size vector);
 
     /**
      * Read bytes from the UART.
@@ -72,7 +71,7 @@ class PL011 : public Device
      *
      * @return Number of bytes on success and ZERO on failure. 
      */
-    virtual Error read(IOBuffer & buffer, Size size, Size offset);
+    virtual FileSystem::Error read(IOBuffer & buffer, Size size, Size offset);
 
     /**
      * Write bytes to the device.
@@ -83,7 +82,7 @@ class PL011 : public Device
      *
      * @return Number of bytes on success and ZERO on failure.
      */
-    virtual Error write(IOBuffer & buffer, Size size, Size offset);
+    virtual FileSystem::Error write(IOBuffer & buffer, Size size, Size offset);
 
   private:
 
