@@ -15,7 +15,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <FreeNOS/System.h>
 #include <TestCase.h>
 #include <TestRunner.h>
 #include <TestInt.h>
@@ -37,7 +36,7 @@ TestCase(RunMpiPing)
 
     // Wait for it to terminate. Exit status must be zero (success)
     pid_t p = waitpid(pid, &status, 0);
-    testAssert(p == pid);
+    testAssert(p == (pid_t) pid);
     testAssert(status == 0);
 
     // Done
